@@ -2,6 +2,7 @@ import websockets
 import websocket
 import json
 import re
+import ssl
 import random
 import argparse
 import sys
@@ -36,7 +37,8 @@ class Bot:
             "advanced",
             "owner",
             "important",
-            "killChildren"]
+            "killChildren",
+            "queue"]
         noneVars = ["userEmail",
             "userPassword",
             "help",
@@ -61,6 +63,7 @@ class Bot:
         self._regexes = {}
         self._copy = False
         self.killChildren = False
+        self.queue = Queue()
         for item in vars:
             if item in kwargs:
                 setattr(self, item, kwargs[item])
@@ -106,7 +109,9 @@ class Bot:
 
     def _recv(self):
         try:
-            return AttrDict(json.loads(self.conn.recv()))
+            msg = AttrDict(json.loads(self.conn.recv()))
+            self.queue.put(msg)
+            return msg
         except json.decoder.JSONDecodeError:
             self.kill()
 
@@ -148,7 +153,11 @@ class Bot:
         if re.search(f'^!send @(?i){re.escape(self.normname)} &(\w+)$', msg.data.content) is not None and self.sendable:
             self.move_to(re.search(f'^!send @(?i){re.escape(self.normname)} &(\w+)$', msg.data.content).group(0))
         for regex, response in self._regexes.items():
-            if re.search(regex, msg.data.content) is not None:
+            if callable(regex):
+                matched = regex(msg)
+            else:
+                matched = re.search(regex, msg.data.content) is not None
+            if matched:
                 if callable(response):
                     result = response(self, msg, re.search(regex, msg.data.content))
                     if isinstance(result, str):
@@ -198,7 +207,7 @@ class Bot:
         sys.stdout.flush()
 
     def connect(self):
-        self.conn = websocket.create_connection(f'wss://euphoria.io/room/{self.room}/ws{"?h=0" if not self.human else "?h=1"}', cookie = self._cookie, enable_multithread = True)
+        self.conn = websocket.create_connection(f'wss://euphoria.io/room/{self.room}/ws{"?h=0" if not self.human else "?h=1"}', sslopt={"cert_reqs": ssl.CERT_NONE}, cookie = self._cookie, enable_multithread = True)
         if self._cookie is None:
             self._cookie = self.conn.headers['set-cookie']
         hello = self._recv()
@@ -227,12 +236,18 @@ class Bot:
             else:
                 self.log("No valid function passed to advanced bot's start function. Using filler function")
                 self.function = lambda msg: None
-            try:
-                while True:
-                    msg = self._recv()
-                    self._handle(msg)
-            except KilledError:
-                pass
+        try:
+            while True:
+                msg = self._recv()
+                self._handle(msg)
+        except KilledError:
+            pass
+
+    def start_in_new_thread(self, function = None, advanced = False):
+        t = Thread(target = self.start, kwargs = {"function": function, "advanced" : advanced}, name = f"{self.nick} in &{self.room}")
+        t.start()
+        t.join(0)
+        return t
 
     def kill(self):
         if self.killChildren:
@@ -350,16 +365,13 @@ class Bot:
             sendable = self.sendable,
             _copy = True,
             _cookie = self._cookie,
-            human = self.human)
+            human = self.human
+            queue = self.queue)
         copy.set_handlers(self.handlers)
         copy.set_regexes(self._regexes)
-        t = Thread(target = copy.move_to, kwargs = {"roomName": roomName, "password" : password, "_copyStart" : True}, name = f"{copy.nick} in &{copy.room}")
         self.children.append(copy)
         copy.parents.append(self)
-        self.childThreads.append(t)
         copy.parentThreads.append(current_thread())
-        t.start()
-        t.join(0)
         return copy
 
     def login(self, email = None, password = None, setDefaults = False):
